<?php

namespace UnicaenDbImport\Service\Facade;

use Doctrine\DBAL\Connection as DbConnection;
use Psr\Log\LoggerAwareTrait;
use UnicaenDbImport\Domain\DestinationInterface;
use UnicaenDbImport\Domain\Exception\ConnectionException;
use UnicaenDbImport\Domain\ResultInterface;
use UnicaenDbImport\Service\Database\DatabaseService;

class LogFacadeService
{
    use LoggerAwareTrait;
    
    /**
     * @var \UnicaenDbImport\Service\Database\DatabaseService
     */
    protected $databaseService;

    /**
     * @var DestinationInterface $destination
     */
    protected $destination;

    /**
     * @var bool
     */
    protected $importLogTableCreated = false;

    /**
     * @param \UnicaenDbImport\Service\Database\DatabaseService $databaseService
     */
    public function __construct(DatabaseService $databaseService)
    {
        $this->databaseService = $databaseService;
    }

    /**
     * @param DestinationInterface $destination
     */
    public function setDestination(DestinationInterface $destination): void
    {
        $this->destination = $destination;
        $this->databaseService->setDestination($this->destination);
    }

    /**
     * @throws \UnicaenDbImport\Service\Exception\DatabaseServiceException
     */
    public function createImportLog()
    {
        if ($this->importLogTableCreated) {
            return;
        }

        $connection = $this->destination->getConnection();

        switch (true) {
            case $connection instanceof DbConnection:
                $this->databaseService->createImportLogTableIfNotExists();
                $this->importLogTableCreated = true;
                break;
            default:
                throw ConnectionException::unexpected($connection);
        }
    }

    /**
     * @param ResultInterface $result
     * @throws \UnicaenDbImport\Service\Exception\DatabaseServiceException
     */
    public function saveResultToImportLog(ResultInterface $result)
    {
        $this->logger->info("Enregistrement des résultats d'import/synchro");

        $connection = $this->destination->getConnection();

        switch (true) {
            case $connection instanceof DbConnection:
                $this->databaseService->saveResultToLogTable($result);
                break;
            default:
                throw ConnectionException::unexpected($connection);
        }
    }
}